/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program 1: GSM-MS Non Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform a simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 *
 * The following measurement example require option CMU-K21 Version >= 2.02
 *
 * The CMU generates a GSM signal which is configured, output via RF 3, fed in
 * via RF 4, and measured with the appropriate analyzer settings. To see which
 * analyzer settings are necessary, it is recommended to carry out the
 * measurement manually first. Before doing so and running the program, connect
 * RF 3 to RF 4 using a coax cable, in analogy to the test setup suggested in
 * chapter 2 of the operating manual.
 *
 * Compiler name and release(s) : LabWindows/CVI 5.0
 * Original Release : January, 2000
 *
 * By ROHDE & SCHWARZ FRANCE:
 *      CONTACTS :
 *              Support & Application
 *              Rohde & Schwarz France
 *              Parc Tertiaire de Meudon
 *              25-27, rue Jeanne Braconnier
 *              92366 Meudon-la-Foret
 *              Telephone : 00 33 1 41 36 10 29
 *              Fax       : 00 33 1 41 36 11 10
 *              e-Mail    : GSA@RSF.RSD.DE
 *
 * Version : 1.2
 *
 * Copyright 2000 ROHDE & SCHWARZ FRANCE. All Rights Reserved.
 *
 * ----------------------------------------------------------------------------
 *                         MODIFICATON HISTORY
 * ----------------------------------------------------------------------------
 *  Date      | Author      | Description
 *    Release |     Company |            
 * ----------------------------------------------------------------------------
 *  31.Mar.00 | S.Blanchon  | Creation (accordingly to the Operating Manual
 *       V1.1 |         RSF | Edition 04).
 *            |             | 
 *            |             | Delivery version 1.1            
 * ----------------------------------------------------------------------------
 *  30.May.00 | S.Blanchon  | Modified accordingly to the splitted CMU
 *       V1.2 |         RSF | LabWindows/CVI Instrument Driver Version 2.03.
 *            |             | 
 *            |             | Delivery version 1.2
 *=============================================================================
 */
#include <utility.h>
#include <ansi_c.h>
#include <string.h>
#include <formatio.h>
#include <visa.h>
#include <userint.h>
#include <cvirte.h> /* Needed if linking in external compiler; harmless otherwise */

// Include R&S CMU driver header file
#include "RSCMU200.h"
#include "RSCMUK2G.h" 

// Variable declaration for secondary address for Base definition and for
// Function Groups definition
static ViSession    g_hRsCmuBase  = VI_NULL;        // Basic Unit
static ViSession    g_hRsCmuRf_NSig = VI_NULL;      // RF Non-Signalling Unit
static ViSession    g_hRsCmuGsm900_Sig = VI_NULL;   // GSM  900 Signalling Unit 
static ViSession    g_hRsCmuGsm900_NSig = VI_NULL;  // GSM  900 Non-Signalling Unit
static ViSession    g_hRsCmuGsm1800_Sig = VI_NULL;  // GSM 1800 Signalling Unit
static ViSession    g_hRsCmuGsm1800_NSig = VI_NULL; // GSM 1800 Non-Signalling Unit

// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif
// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
    ViChar l_vCmuErrMsg[256];
    ViChar l_vErrBuf[512];
    ViInt32 _error;
    
    if (CmuStatus != VI_SUCCESS)
    {
        RSCMU_K2G_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
        Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
        MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
        return -1;
    }
    return 0;
    
Error:
    return -1;
}

// Macro "WriteCmuViInt32Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViInt32Value
#define WriteCmuViInt32Value(String,ViInt32Value) \
(ViInt32Value != RSCMU_NAN_VI_INT32) ? printf ("%s%d\n", String, ViInt32Value) : printf ("%sNot a number\n", String);
#endif

// Macro "WriteCmuViReal64Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViReal64Value
#define WriteCmuViReal64Value(String,ViReal64Value) \
(ViReal64Value != RSCMU_NAN) ? printf ("%s%lf\n", String, ViReal64Value) : printf ("%sNot a number\n", String);
#endif

/*****************************************************************************/
/*======================== Main =============================================*/
/*****************************************************************************/
int main (int argc, char *argv[])
{                                  
  ViInt32    _error;
  
  ViChar     l_vInstrDriverRev[256];
  ViChar     l_vInstrFirmwareRev[256];
  
  ViChar     errorMessage[256];
  
  ViReal64   l_vAvgBurstPwCurr;
  ViReal64   l_vPeakBurstPwCurr;
  ViReal64   l_vBurstsOutOfPowTol;
  ViInt32    l_vBurstMatching;
  ViReal64   l_vAvgBurstPwAvg;
  
  ViReal64   l_vPhErrPeakMaxMin;
  ViReal64   l_vPhErrRMSMaxMin;
  ViReal64   l_vFreqErrMaxMin;
  
  ViReal64   l_vPkPwr;

  if (InitCVIRTE (0, argv, 0) == 0) /* Needed if linking in external compiler; harmless otherwise */
    return -1; /* out of memory */
  
  Cls();
  
  
  // ==========================================================================
  // CMU BASIC UNIT
  // --------------------------------------------------------------------------
  // BASIC UNIT INITIALISATION
  //  -> GPIB board index is assigned to 0
  //  -> Primary address to the GPIB bus is assigned to the default (20)
  //  -> Ask for the indentifier of the CMU, reset the instrument
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_Base (&g_hRsCmuBase, RSCMU_GPIB, 0, 20, RSCMU_DO_QUERY, RSCMU_RESET_ON, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS CONFIGURATION
  //   Subsystem Remote
  //     -> Secondary addresses of the GPIB bus are assigned for all available
  //        function groups
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 1, RSCMU_RF_NSIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 2, RSCMU_GSM900MS_SIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 3, RSCMU_GSM900MS_NSIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 4, RSCMU_GSM1800MS_SIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 5, RSCMU_GSM1800MS_NSIG, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS INITIALISATION
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_RF_NSig (g_hRsCmuBase, &g_hRsCmuRf_NSig, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_Sig (g_hRsCmuBase, &g_hRsCmuGsm900_Sig, RSCMU_GSM900, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_NSig (g_hRsCmuBase, &g_hRsCmuGsm900_NSig, RSCMU_GSM900, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_Sig (g_hRsCmuBase, &g_hRsCmuGsm1800_Sig, RSCMU_GSM1800, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_NSig (g_hRsCmuBase, &g_hRsCmuGsm1800_NSig, RSCMU_GSM1800, errorMessage));
  
  // --------------------------------------------------------------------------
  //     -> Set a specified timeout value for all available function groups.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm900_Sig, 10000, errorMessage));
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm900_NSig, 10000, errorMessage));
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm1800_Sig, 10000, errorMessage));
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm1800_NSig, 10000, errorMessage));
  
  CheckCmuError (RSCMU_K2G_RevisionQuery (g_hRsCmuBase, l_vInstrDriverRev, l_vInstrFirmwareRev, errorMessage));
  
  // --------------------------------------------------------------------------
  printf ("Instrument Driver Revision  (K2G) : %s\n", l_vInstrDriverRev);
  printf ("Instrument Firmware Revision      : %s\n", l_vInstrFirmwareRev);
  printf ("\n");
  
  // Opens the file "RsCmu.iee" for output IEEE bus commands.
  CheckCmuError (RSCMU_Trace_ON (g_hRsCmuBase, "RsCmu.iee", errorMessage));
  
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // CONNECTORS + ATTENUATION
  //   Define input connector RF4 and output connector RF3.
  //   Make sure that external attenuations are set to zero.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_Inp (g_hRsCmuGsm900_NSig, RSCMU_INPUT_RF4, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_Outp (g_hRsCmuGsm900_NSig, RSCMU_OUTPUT_RF3, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_SensCorrLossInp (g_hRsCmuGsm900_NSig, RSCMU_INPUT_RF4, 0.0, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_SourCorrLossOutp (g_hRsCmuGsm900_NSig, RSCMU_OUTPUT_RF3, 0.0, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // GENERATOR SETTINGS
  //   Select GSM dummy burst to be generated.
  //   Select training sequence.
  //   Select signal shape (burst signal).
  //   Set RF carrier frequency to 900.0 MHz
  //   Set RF level in used timeslot to -5.0 dBm.
  //   And switch on RF generator.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_ConfRfgModBitSel (g_hRsCmuGsm900_NSig, RSCMU_DUMM, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfRfgModTseqSel (g_hRsCmuGsm900_NSig, RSCMU_GSM0, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfRfgModTran (g_hRsCmuGsm900_NSig, RSCMU_TRAN_BURS, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_SourRfgFreq (g_hRsCmuGsm900_NSig, 900.0, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_SourRfgLevUtim (g_hRsCmuGsm900_NSig, -5.0, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_InitRfg (g_hRsCmuGsm900_NSig, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // ANALYZER SETTINGS
  //   Adjust analyzer frequency to generator frequency (900.0 MHz)
  //   Adjust training sequence analyzed (GSM0)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_RfanFreq (g_hRsCmuGsm900_NSig, 900.0, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_RfanTseq (g_hRsCmuGsm900_NSig, RSCMU_GSM0, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // CONFIGURE THE POWER MEASUREMENT
  //   Scalar results, one burst per statistics cycle.
  //   Single shot measurement, no stop on error.
  //   Same settings as before, but for combined power and modulation measurement
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_ConfPowCont (g_hRsCmuGsm900_NSig, RSCMU_SCALAR, 1, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfPowContRep (g_hRsCmuGsm900_NSig, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfPowMprCont (g_hRsCmuGsm900_NSig, RSCMU_SCALAR, 1, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfPowMprContRep (g_hRsCmuGsm900_NSig, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SETTLE THE EXPECTED POWER (+OVERSHOOT MARGIN) AND MEASURE THE POWER
  // REMEMBER: RFG POWER -5.0 dBm
  //   Adjust expected maximum level to 0.0 dBm
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_SensLevMax (g_hRsCmuGsm900_NSig, 0.0, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Frequency 900.0 MHz RF Generator Power -5.0 dBm
  //
  // Start single shot measurement, wait until it is terminated, and return
  // scalar power and modulation results:
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  printf ("\tRF Generator: 900.0 MHz,  -5.0 dBm\n");
  printf ("\t----------------------------------\n");
  
  CheckCmuError (RSCMU_GSM_NSig_ReadPowMpr (g_hRsCmuGsm900_NSig,
                                            RSCMU_MAXMIN,
                                            &l_vAvgBurstPwCurr,
                                            &l_vPeakBurstPwCurr,
                                            &l_vBurstsOutOfPowTol,
                                            &l_vBurstMatching,
                                            &l_vPhErrPeakMaxMin,
                                            &l_vPhErrRMSMaxMin,
                                            &l_vFreqErrMaxMin,
                                            &l_vAvgBurstPwAvg,
                                            errorMessage));
  
  WriteCmuViReal64Value ("\tAvg. Burst Power (Cur.)[dBm] = ", l_vAvgBurstPwCurr);
  WriteCmuViReal64Value ("\tPeak Burst Power (Cur.)[dBm] = ", l_vPeakBurstPwCurr);
  WriteCmuViReal64Value ("\tAvg. Burst Power (Avg.)[dBm] = ", l_vAvgBurstPwAvg);
  WriteCmuViInt32Value  ("\tBurst Power Matching         = ", l_vBurstMatching);
  WriteCmuViReal64Value ("\tMaximum phase error peak [] = ", l_vPhErrPeakMaxMin);
  WriteCmuViReal64Value ("\tMaximum phase error RMS  [] = ", l_vPhErrRMSMaxMin);
  WriteCmuViReal64Value ("\tMaximum frequecy error  [Hz] = ", l_vFreqErrMaxMin);
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // FREE TX MEASUREMENT DSP RESOURCES
  //   Abort combined power/mod measurement, free resources.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_AborPowMpr (g_hRsCmuGsm900_NSig, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // RFG POWER -10 dBm, USING FASTER WIDEBAND PEAK POWER METER
  //   Select RF level in used timeslot to -10.0 dBm.
  //   Adjust expected maximum level to -5.0 dBm.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_NSig_SourRfgLevUtim (g_hRsCmuGsm900_NSig, -10.0, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_SensLevMax (g_hRsCmuGsm900_NSig, -5.0, errorMessage));
  
  printf ("\n");
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Frequency 900.0 MHz RF Generator Power -10.0 dBm
  //
  // Start single shot wide band peak power measurement, wait until it is
  // terminated, and return result
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  printf ("\tRF Generator: 900.0 MHz, -10.0 dBm\n");
  printf ("\t----------------------------------\n");
  
  CheckCmuError (RSCMU_GSM_NSig_ReadSpow (g_hRsCmuGsm900_NSig, &l_vPkPwr, errorMessage));
  
  WriteCmuViReal64Value ("\tPeak Burst Power       [dBm] = ", l_vPkPwr);
  
  // The measurement can be easily repeated at other RF levels or with different configurations.
  
  // Close the file "RsCmu.iee".
  CheckCmuError (RSCMU_Trace_OFF (g_hRsCmuBase, errorMessage));
  
  CheckCmuError (RSCMU_Close_RF_NSig (g_hRsCmuRf_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_Sig (g_hRsCmuGsm900_Sig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_NSig (g_hRsCmuGsm900_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_Sig (g_hRsCmuGsm1800_Sig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_NSig (g_hRsCmuGsm1800_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_Base (g_hRsCmuBase, errorMessage));

  return (0);

Error:
  return (-1);
}
